;
; Definition file of libeac-0.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "libeac-1.dll"
EXPORTS
ALGORITHM_IDENTIFIER_it
BN_bn2buf
BUF_MEM_clear_free
BUF_MEM_create
BUF_MEM_create_init
BUF_MEM_dup
BUF_MEM_print
CARD_INFO_LOCATOR_it
CA_CTX_clear_free
CA_CTX_new
CA_CTX_set_protocol
CA_DP_INFO_free
CA_DP_INFO_it
CA_DP_INFO_new
CA_INFO_free
CA_INFO_it
CA_INFO_new
CA_PUBLIC_KEY_INFO_free
CA_PUBLIC_KEY_INFO_it
CA_PUBLIC_KEY_INFO_new
CA_STEP1_get_pubkey
CA_STEP2_get_eph_pubkey
CA_STEP3_check_pcd_pubkey
CA_STEP4_compute_shared_secret
CA_STEP5_derive_keys
CA_STEP6_derive_keys
CA_disable_passive_authentication
CA_get_pubkey
CA_set_key
CVC_CERTIFICATE_DESCRIPTION_free
CVC_CERTIFICATE_DESCRIPTION_it
CVC_CERTIFICATE_DESCRIPTION_new
CVC_CERTIFICATE_DESCRIPTION_print_ctx
CVC_CERT_AUTHENTICATION_REQUEST_SEQ_it
CVC_CERT_AUTHENTICATION_REQUEST_free
CVC_CERT_AUTHENTICATION_REQUEST_it
CVC_CERT_AUTHENTICATION_REQUEST_new
CVC_CERT_AUTHENTICATION_REQUEST_print_ctx
CVC_CERT_BODY_SEQ_it
CVC_CERT_BODY_free
CVC_CERT_BODY_it
CVC_CERT_BODY_new
CVC_CERT_REQUEST_BODY_SEQ_it
CVC_CERT_REQUEST_BODY_free
CVC_CERT_REQUEST_BODY_it
CVC_CERT_REQUEST_BODY_new
CVC_CERT_REQUEST_SEQ_it
CVC_CERT_REQUEST_free
CVC_CERT_REQUEST_it
CVC_CERT_REQUEST_new
CVC_CERT_REQUEST_print_ctx
CVC_CERT_SEQ_it
CVC_CERT_free
CVC_CERT_it
CVC_CERT_new
CVC_CERT_print_ctx
CVC_CHAT_SEQ_it
CVC_CHAT_free
CVC_CHAT_it
CVC_CHAT_new
CVC_CHAT_print_ctx
CVC_COMMCERT_SEQ_it
CVC_DISCRETIONARY_DATA_TEMPLATE_SEQ_it
CVC_DISCRETIONARY_DATA_TEMPLATE_free
CVC_DISCRETIONARY_DATA_TEMPLATE_it
CVC_DISCRETIONARY_DATA_TEMPLATE_new
CVC_PUBKEY_BODY_it
CVC_PUBKEY_free
CVC_PUBKEY_it
CVC_PUBKEY_new
CVC_check_description
CVC_d2i_CVC_CERT
CVC_get_car
CVC_get_chr
CVC_get_effective_date
CVC_get_expiration_date
CVC_get_profile_identifier
CVC_get_role
CVC_hash_description
CVC_pkey2pubkey
CVC_print
CVC_pubkey2eckey
CVC_pubkey2pkey
CVC_verify_authentication_request_signatures
CVC_verify_request_signature
CVC_verify_signature
Comp
DH_get_order
DH_get_q
DHparams_dup_with_q
EAC_CTX_clear_free
EAC_CTX_get_csca_lookup
EAC_CTX_get_cvca_lookup
EAC_CTX_init_ca
EAC_CTX_init_ef_cardaccess
EAC_CTX_init_ef_cardsecurity
EAC_CTX_init_pace
EAC_CTX_init_ri
EAC_CTX_init_ta
EAC_CTX_new
EAC_CTX_print_private
EAC_CTX_set_csca_lookup
EAC_CTX_set_cvca_lookup
EAC_CTX_set_encryption_ctx
EAC_Comp
EAC_add_all_objects
EAC_add_iso_pad
EAC_authenticate
EAC_cleanup
EAC_decrypt
EAC_encrypt
EAC_get_default_csca_lookup
EAC_get_default_cvca_lookup
EAC_hash_certificate_description
EAC_increment_ssc
EAC_init
EAC_remove_iso_pad
EAC_reset_ssc
EAC_set_cvc_default_dir
EAC_set_ssc
EAC_set_x509_default_dir
EAC_sign
EAC_verify
EAC_verify_authentication
ECDH_OpenSSL_Point
EC_POINT_point2mem
EVP_PKEY_dup
EVP_PKEY_set_keys
EVP_PKEY_set_std_dp
FILE_ID_it
KA_CTX_clear_free
KA_CTX_compute_key
KA_CTX_derive_keys
KA_CTX_dup
KA_CTX_generate_key
KA_CTX_new
KA_CTX_set_protocol
NID_cardInfoLocator DATA
NID_eID DATA
NID_ecka_dh_SessionKDF DATA
NID_ecka_dh_SessionKDF_AES128 DATA
NID_ecka_dh_SessionKDF_AES192 DATA
NID_ecka_dh_SessionKDF_AES256 DATA
NID_ecka_dh_SessionKDF_DES3 DATA
NID_id_AT DATA
NID_id_CA_DH DATA
NID_id_CA_DH_3DES_CBC_CBC DATA
NID_id_CA_DH_AES_CBC_CMAC_128 DATA
NID_id_CA_DH_AES_CBC_CMAC_192 DATA
NID_id_CA_DH_AES_CBC_CMAC_256 DATA
NID_id_CA_ECDH DATA
NID_id_CA_ECDH_3DES_CBC_CBC DATA
NID_id_CA_ECDH_AES_CBC_CMAC_128 DATA
NID_id_CA_ECDH_AES_CBC_CMAC_192 DATA
NID_id_CA_ECDH_AES_CBC_CMAC_256 DATA
NID_id_CI DATA
NID_id_CommunityID DATA
NID_id_DateOfBirth DATA
NID_id_DateOfExpiry DATA
NID_id_IS DATA
NID_id_PACE_DH_GM DATA
NID_id_PACE_DH_GM_3DES_CBC_CBC DATA
NID_id_PACE_DH_GM_AES_CBC_CMAC_128 DATA
NID_id_PACE_DH_GM_AES_CBC_CMAC_192 DATA
NID_id_PACE_DH_GM_AES_CBC_CMAC_256 DATA
NID_id_PACE_DH_IM DATA
NID_id_PACE_DH_IM_3DES_CBC_CBC DATA
NID_id_PACE_DH_IM_AES_CBC_CMAC_128 DATA
NID_id_PACE_DH_IM_AES_CBC_CMAC_192 DATA
NID_id_PACE_DH_IM_AES_CBC_CMAC_256 DATA
NID_id_PACE_ECDH_GM DATA
NID_id_PACE_ECDH_GM_3DES_CBC_CBC DATA
NID_id_PACE_ECDH_GM_AES_CBC_CMAC_128 DATA
NID_id_PACE_ECDH_GM_AES_CBC_CMAC_192 DATA
NID_id_PACE_ECDH_GM_AES_CBC_CMAC_256 DATA
NID_id_PACE_ECDH_IM DATA
NID_id_PACE_ECDH_IM_3DES_CBC_CBC DATA
NID_id_PACE_ECDH_IM_AES_CBC_CMAC_128 DATA
NID_id_PACE_ECDH_IM_AES_CBC_CMAC_192 DATA
NID_id_PACE_ECDH_IM_AES_CBC_CMAC_256 DATA
NID_id_PK_DH DATA
NID_id_PK_ECDH DATA
NID_id_PT DATA
NID_id_RI_DH DATA
NID_id_RI_DH_SHA_1 DATA
NID_id_RI_DH_SHA_224 DATA
NID_id_RI_DH_SHA_256 DATA
NID_id_RI_DH_SHA_384 DATA
NID_id_RI_DH_SHA_512 DATA
NID_id_RI_ECDH DATA
NID_id_RI_ECDH_SHA_1 DATA
NID_id_RI_ECDH_SHA_224 DATA
NID_id_RI_ECDH_SHA_256 DATA
NID_id_RI_ECDH_SHA_384 DATA
NID_id_RI_ECDH_SHA_512 DATA
NID_id_ST DATA
NID_id_SecurityObject DATA
NID_id_TA DATA
NID_id_TA_ECDSA DATA
NID_id_TA_ECDSA_SHA_1 DATA
NID_id_TA_ECDSA_SHA_224 DATA
NID_id_TA_ECDSA_SHA_256 DATA
NID_id_TA_ECDSA_SHA_384 DATA
NID_id_TA_ECDSA_SHA_512 DATA
NID_id_TA_RSA DATA
NID_id_TA_RSA_PSS_SHA_1 DATA
NID_id_TA_RSA_PSS_SHA_256 DATA
NID_id_TA_RSA_PSS_SHA_512 DATA
NID_id_TA_RSA_v1_5_SHA_1 DATA
NID_id_TA_RSA_v1_5_SHA_256 DATA
NID_id_TA_RSA_v1_5_SHA_512 DATA
NID_id_description DATA
NID_id_eIDSecurity DATA
NID_id_htmlFormat DATA
NID_id_pdfFormat DATA
NID_id_plainFormat DATA
NID_id_sector DATA
NID_standardizedDomainParameters DATA
PACE_CTX_clear_free
PACE_CTX_new
PACE_CTX_set_protocol
PACE_DP_INFO_free
PACE_DP_INFO_it
PACE_DP_INFO_new
PACE_INFO_free
PACE_INFO_it
PACE_INFO_new
PACE_SEC_clear_free
PACE_SEC_new
PACE_SEC_print_private
PACE_STEP1_enc_nonce
PACE_STEP2_dec_nonce
PACE_STEP3A_generate_mapping_data
PACE_STEP3A_map_generator
PACE_STEP3B_compute_shared_secret
PACE_STEP3B_generate_ephemeral_key
PACE_STEP3C_derive_keys
PACE_STEP3D_compute_authentication_token
PACE_STEP3D_verify_authentication_token
PROTOCOL_PARAMS_it
RI_CTX_clear_free
RI_CTX_new
RI_CTX_set_protocol
RI_DP_INFO_free
RI_DP_INFO_it
RI_DP_INFO_new
RI_INFO_free
RI_INFO_it
RI_INFO_new
RI_STEP2_compute_identifier
SUBJECT_PUBLIC_KEY_INFO_it
TA_CTX_clear_free
TA_CTX_import_certificate
TA_CTX_new
TA_CTX_set_trust_anchor
TA_INFO_free
TA_INFO_it
TA_INFO_new
TA_STEP2_import_certificate
TA_STEP3_generate_ephemeral_key
TA_STEP4_get_nonce
TA_STEP4_set_nonce
TA_STEP5_sign
TA_STEP6_verify
TA_disable_checks
add_iso_pad
asn1_pubkey
authenticate
certificate_authentication_request_print
certificate_description_print
certificate_request_print
cipher_no_pad
cmac
cvc_chat_print
cvc_chat_print_authorizations
cvc_get_chat
cvc_get_date_string
d2i_CA_DP_INFO
d2i_CA_INFO
d2i_CA_PUBLIC_KEY_INFO
d2i_CVC_CERT
d2i_CVC_CERTIFICATE_DESCRIPTION
d2i_CVC_CERT_AUTHENTICATION_REQUEST
d2i_CVC_CERT_BODY
d2i_CVC_CERT_REQUEST
d2i_CVC_CERT_REQUEST_BODY
d2i_CVC_CERT_bio
d2i_CVC_CHAT
d2i_CVC_DISCRETIONARY_DATA_TEMPLATE
d2i_CVC_PUBKEY
d2i_PACE_DP_INFO
d2i_PACE_INFO
d2i_RI_DP_INFO
d2i_RI_INFO
d2i_TA_INFO
dh_compute_key
dh_generate_key
dh_gm_compute_key
dh_gm_generate_key
dh_im_compute_key
dh_im_generate_key
eac_oid2md
ecdh_compute_key
ecdh_generate_key
ecdh_gm_compute_key
ecdh_gm_generate_key
ecdh_im_compute_key
ecdh_im_generate_key
encode_ssc
get_authentication_token
get_pubkey
hash
i2d_CA_DP_INFO
i2d_CA_INFO
i2d_CA_PUBLIC_KEY_INFO
i2d_CVC_CERT
i2d_CVC_CERTIFICATE_DESCRIPTION
i2d_CVC_CERT_AUTHENTICATION_REQUEST
i2d_CVC_CERT_BODY
i2d_CVC_CERT_REQUEST
i2d_CVC_CERT_REQUEST_BODY
i2d_CVC_CHAT
i2d_CVC_DISCRETIONARY_DATA_TEMPLATE
i2d_CVC_PUBKEY
i2d_PACE_DP_INFO
i2d_PACE_INFO
i2d_RI_DP_INFO
i2d_RI_INFO
i2d_TA_INFO
init_dh
init_ecdh
is_bcd
is_char_str
is_chr
kdf
kdf_enc
kdf_mac
kdf_pi
randb
retail_mac_des
update_iv
verify_authentication_token
